################################################################################
#
#   Purchase requisition reports module for OpenERP
#   Copyright (C) 2009, Almacom (Thailand) Ltd.
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
################################################################################

from osv import fields,osv

class report_pr_month(osv.osv):
    _name="report.pr.month"
    _auto=False
    _columns={
        "name":fields.char("Month",size=64,readonly=True,select=True),
        "year":fields.char("Year",size=64,readonly=True,select=True),
        "state":fields.selection([("confirmed","Confirmed"),("done","Delivered")],"State",readonly=True,select=True),
        "price_total":fields.float("Total Price",readonly=True),
        "department_id":fields.many2one("hr.department","Department",readonly=True),
    }
    _order="name desc"

    def init(self,cr):
        cr.execute("""
create or replace view report_pr_month as (
    select
        min(l.id) as id,
        to_char(pr.date_confirmed,'YYYY-MM') as name,
        to_char(pr.date_confirmed,'YYYY') as year,
        pr.state,
        sum(l.qty*l.unit_price_confirmed)+0.001 as price_total,
        pr.department_id
    from purchase_requisition pr
        left join purchase_requisition_line l on (pr.id=l.order_id)
    where pr.state in ('confirmed','done') and l.product_id is not null
    group by to_char(pr.date_confirmed,'YYYY-MM'),to_char(pr.date_confirmed,'YYYY'),pr.state,pr.department_id
    order by price_total limit 20
)""")
report_pr_month()

class report_pr_week(osv.osv):
    _name="report.pr.week"
    _auto=False
    _columns={
        "name":fields.char("Week",size=64,readonly=True,select=True),
        "state":fields.selection([("confirmed","Confirmed"),("done","Delivered")],"State",readonly=True,select=True),
        "price_total":fields.float("Total Price",readonly=True),
        "department_id":fields.many2one("hr.department","Department",readony=True),
    }
    _order="name desc"

    def init(self,cr):
        cr.execute("""
create or replace view report_pr_week as (
    select
        min(l.id) as id,
        to_char(pr.date_confirmed,'YYYY-WW') as name,
        pr.state,
        sum(l.qty*l.unit_price_confirmed)+0.001 as price_total,
        pr.department_id
    from purchase_requisition pr
        left join purchase_requisition_line l on (pr.id=l.order_id)
    where pr.state in ('confirmed','done') and l.product_id is not null
    group by to_char(pr.date_confirmed,'YYYY-WW'),pr.state,pr.department_id
    order by price_total limit 20
)""")
report_pr_week()

class hr_department(osv.osv):
    _inherit="hr.department"

    def _pr_month(self,cr,uid,ids,name,args,context={}):
        vals={}
        for dept_id in ids:
            cr.execute("select sum(prl.qty*prl.unit_price_confirmed)+0.001 from purchase_requisition pr left join purchase_requisition_line prl on prl.order_id=pr.id where pr.department_id=%s and pr.state in ('confirmed','done') and date_confirmed>=date_trunc('month',date 'today')",(dept_id,))
            r=cr.fetchone()
            vals[dept_id]=r[0]
        return vals

    def _pr_year(self,cr,uid,ids,name,args,context={}):
        vals={}
        for dept_id in ids:
            cr.execute("select sum(prl.qty*prl.unit_price_confirmed)+0.001 from purchase_requisition pr left join purchase_requisition_line prl on prl.order_id=pr.id where pr.department_id=%s and pr.state in ('confirmed','done') and date_confirmed>=date_trunc('year',date 'today')",(dept_id,))
            r=cr.fetchone()
            vals[dept_id]=r[0]
        return vals

    _columns={
        "pr_month": fields.function(_pr_month,method=True,type="float",string="PR This Month"),
        "pr_year": fields.function(_pr_year,method=True,type="float",string="PR This Year"),
    }
hr_department()
